<?php

declare(strict_types=1);

namespace Dedi\SyliusMigrationPlugin\Config;

use Dedi\SyliusMigrationPlugin\Definition\ColumnMappingDefinitionFactory;
use Dedi\SyliusMigrationPlugin\Definition\TableMappingDefinition;
use Dedi\SyliusMigrationPlugin\Dto\AfterMigrateDefinition;
use Dedi\SyliusMigrationPlugin\Dto\FilterMappingDefinition;
use Dedi\SyliusMigrationPlugin\Dto\IdentifierMappingDefinition;
use Dedi\SyliusMigrationPlugin\Dto\LimitMappingDefinition;
use Dedi\SyliusMigrationPlugin\Dto\OrderMappingDefinition;
use Dedi\SyliusMigrationPlugin\Dto\SearchMappingDefinition;
use Doctrine\Common\Collections\ArrayCollection;

class MigrationMappingConfig
{
    private array $tableDefinitions = [];

    private readonly ColumnMappingDefinitionFactory $columnMappingDefinitionFactory;

    public function __construct(ColumnMappingDefinitionFactory $columnMappingDefinitionFactory)
    {
        $this->columnMappingDefinitionFactory = $columnMappingDefinitionFactory;
    }

    public function addTableMappingDefinition(
        string $targetTableName,
        string $sourceTableName,
        string $sql,
        string $sourcePrimaryKey,
        array $identifierMapping,
        array $columnDefinitions,
        array $search,
        array $filter,
        array $logginColumns,
        array $limit,
        ?array $order = null,
        ?array $afterMigrate = null,
        ?string $handler = null,
        ?array $errorHandlers = null,
    ): void {
        $columnMappingDefinitions = new ArrayCollection();
        foreach ($columnDefinitions as $column => $definition) {
            $columnMappingDefinitions->add($this->columnMappingDefinitionFactory->createFromDefinition($column, $definition));
        }
        $identifierMappingDefinitions = new ArrayCollection();
        foreach ($identifierMapping as $definition) {
            if (!array_key_exists('source_table', $definition)) {
                $definition['source_table'] = $sourceTableName;
            }
            if (!array_key_exists('dest_table', $definition)) {
                $definition['dest_table'] = $targetTableName;
            }
            $identifierMappingDefinitions->add(IdentifierMappingDefinition::fromArray($definition));
        }

        $searchMappingDefinitions = new ArrayCollection();
        foreach ($search as $definition) {
            $searchMappingDefinitions->add(SearchMappingDefinition::fromArray($definition));
        }

        $filterMappingDefinitions = new ArrayCollection();
        foreach ($filter as $definition) {
            $filterMappingDefinitions->add(FilterMappingDefinition::fromArray($definition));
        }

        $tableMappingDefinition = new TableMappingDefinition(
            $targetTableName,
            $sourceTableName,
            $sql,
            $sourcePrimaryKey,
            $identifierMappingDefinitions->toArray(),
            $columnMappingDefinitions->toArray(),
            $searchMappingDefinitions->toArray(),
            $filterMappingDefinitions->toArray(),
            $logginColumns,
            LimitMappingDefinition::fromArray($limit),
            null !== $order ? OrderMappingDefinition::fromArray($order) : null,
            null !== $afterMigrate ? AfterMigrateDefinition::fromArray($afterMigrate) : null,
            $handler,
            $errorHandlers,
        );

        $this->tableDefinitions[] = $tableMappingDefinition;
    }

    /** @return iterable|TableMappingDefinition[] */
    public function getTableDefinitions(): iterable
    {
        return $this->tableDefinitions;
    }
}
