<?php

declare(strict_types=1);

namespace Dedi\SyliusMigrationPlugin\Registry;

use Dedi\SyliusMigrationPlugin\Service\Transformer\TransformerInterface;
use LogicException;

class TransformerRegistry
{
    private array $transformers = [];

    public function getTransformers(): array
    {
        return $this->transformers;
    }

    public function addTransformer(TransformerInterface $transformer): array
    {
        if (!$this->hasTransformer($transformer->getType())) {
            $this->transformers[$transformer->getType()] = $transformer;
        }

        return $this->transformers;
    }

    public function getTransformer(string $type): TransformerInterface
    {
        if (!$this->hasTransformer($type)) {
            throw new LogicException(sprintf('Transformer %s not registered', $type));
        }

        return $this->transformers[$type];
    }

    public function hasTransformer(string $type): bool
    {
        return array_key_exists($type, $this->transformers);
    }
}
